/* GLSL_parallax: Parallax mapping with with GLSL shaders
   Copyright (C) 2007 Angelo "Encelo" Theodorou
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glext.h> 

#include "init.h"

void UpdateLight(struct Light *l)
{
	glLightfv(GL_LIGHT0, GL_POSITION, l->position);
	glLightfv(GL_LIGHT0, GL_AMBIENT, l->ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, l->diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, l->specular);

	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);

	glColor3f(l->diffuse[0], l->diffuse[1], l->diffuse[2]);
	glPushMatrix();
	glBegin(GL_QUADS);
		glVertex3f(l->position[0]+0.025f, l->position[1]+0.025f, l->position[2]);
		glVertex3f(l->position[0]-0.025f, l->position[1]+0.025f, l->position[2]);
		glVertex3f(l->position[0]-0.025f, l->position[1]-0.025f, l->position[2]);
		glVertex3f(l->position[0]+0.025f, l->position[1]-0.025f, l->position[2]);
	glEnd();
	glPopMatrix();

	glPopAttrib();
}


void DrawCube(int withBuffers, GLuint vboid[])
{
	if (withBuffers) {	
		glEnableClientState(GL_VERTEX_ARRAY);
		glBindBuffer(GL_ARRAY_BUFFER, vboid[0]);
		glVertexPointer(3, GL_FLOAT, 0, NULL);

		glClientActiveTexture(GL_TEXTURE0);
		glBindBuffer(GL_ARRAY_BUFFER, vboid[1]);
		glTexCoordPointer(2, GL_FLOAT, 0, NULL);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);

		glClientActiveTexture(GL_TEXTURE1);
		glBindBuffer(GL_ARRAY_BUFFER, vboid[1]);
		glTexCoordPointer(2, GL_FLOAT, 0, NULL);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);

		glClientActiveTexture(GL_TEXTURE2);
		glBindBuffer(GL_ARRAY_BUFFER, vboid[1]);
		glTexCoordPointer(2, GL_FLOAT, 0, NULL);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);

		glBindBuffer(GL_ARRAY_BUFFER, vboid[2]);
		glNormalPointer(GL_FLOAT, 0, NULL);
		glEnableClientState(GL_NORMAL_ARRAY);

		glClientActiveTexture(GL_TEXTURE3);
		glBindBuffer(GL_ARRAY_BUFFER, vboid[3]);
		glTexCoordPointer(3, GL_FLOAT, 0, NULL);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);

		glDrawArrays(GL_QUADS, 0, 24);
	}
	else {
		glClientActiveTexture(GL_TEXTURE3);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glDisableClientState(GL_NORMAL_ARRAY);
		glClientActiveTexture(GL_TEXTURE2);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glClientActiveTexture(GL_TEXTURE1);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glClientActiveTexture(GL_TEXTURE0);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glDisableClientState(GL_VERTEX_ARRAY);

		glBegin(GL_QUADS);
		// Front Face
		glNormal3f( 0.0f, 0.0f, 1.0f);					// Normal Pointing Towards Viewer
		glMultiTexCoord3f(GL_TEXTURE3,-1.0f, 0.0f, 0.0f); // Tangent
		glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 1.0f, 1.0f);
		glVertex3f(-1.0f, -1.0f,  1.0f);	// Point 1 (Front)
		glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 0.0f, 1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);	// Point 2 (Front)
		glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 0.0f, 0.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);	// Point 3 (Front)
		glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 1.0f, 0.0f);
		glVertex3f(-1.0f,  1.0f,  1.0f);	// Point 4 (Front)
		// Back Face
		glNormal3f(0.0f, 0.0f,-1.0f);					// Normal Pointing Away From Viewer
		glMultiTexCoord3f(GL_TEXTURE3, 1.0f, 0.0f, 0.0f); // Tangent
		glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 0.0f, 1.0f);
		glVertex3f(-1.0f, -1.0f, -1.0f);	// Point 1 (Back)
		glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 0.0f, 0.0f);
		glVertex3f(-1.0f,  1.0f, -1.0f);	// Point 2 (Back)
		glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 1.0f, 0.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);	// Point 3 (Back)
		glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 1.0f, 1.0f);
		glVertex3f( 1.0f, -1.0f, -1.0f);	// Point 4 (Back)
		// Top Face
		glNormal3f(0.0f, 1.0f, 0.0f);					// Normal Pointing Up
		glMultiTexCoord3f(GL_TEXTURE3,-1.0f, 0.0f, 0.0f); // Tangent
		glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 1.0f, 0.0f);
		glVertex3f(-1.0f,  1.0f, -1.0f);	// Point 1 (Top)
		glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 1.0f, 1.0f);
		glVertex3f(-1.0f,  1.0f,  1.0f);	// Point 2 (Top)
		glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 0.0f, 1.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);	// Point 3 (Top)
		glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 0.0f, 0.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);	// Point 4 (Top)
		// Bottom Face
		glNormal3f(0.0f,-1.0f, 0.0f);					// Normal Pointing Down
		glMultiTexCoord3f(GL_TEXTURE3, 1.0f, 0.0f, 0.0f); // Tangent
		glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 0.0f, 0.0f);
		glVertex3f(-1.0f, -1.0f, -1.0f);	// Point 1 (Bottom)
		glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 1.0f, 0.0f);
		glVertex3f( 1.0f, -1.0f, -1.0f);	// Point 2 (Bottom)
		glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 1.0f);
	   	glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 1.0f, 1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);	// Point 3 (Bottom)
		glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 0.0f, 1.0f);
		glVertex3f(-1.0f, -1.0f,  1.0f);	// Point 4 (Bottom)
		// Right face
		glNormal3f(1.0f, 0.0f, 0.0f);					// Normal Pointing Right
		glMultiTexCoord3f(GL_TEXTURE3, 0.0f, 0.0f, 1.0f); // Tangent
		glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 0.0f, 1.0f);
		glVertex3f( 1.0f, -1.0f, -1.0f);	// Point 1 (Right)
		glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 0.0f, 0.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);	// Point 2 (Right)
		glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 1.0f, 0.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);	// Point 3 (Right)
		glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 1.0f, 1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);	// Point 4 (Right)
		// Left Face
		glNormal3f(-1.0f, 0.0f, 0.0f);					// Normal Pointing Left
		glMultiTexCoord3f(GL_TEXTURE3, 0.0f, 0.0f,-1.0f); // Tangent
		glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 1.0f, 1.0f);
		glVertex3f(-1.0f, -1.0f, -1.0f);	// Point 1 (Left)
		glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 1.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 0.0f, 1.0f);
		glVertex3f(-1.0f, -1.0f,  1.0f);	// Point 2 (Left)
		glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 0.0f, 0.0f);
		glVertex3f(-1.0f,  1.0f,  1.0f);	// Point 3 (Left)
		glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 0.0f);
		glMultiTexCoord2f(GL_TEXTURE2, 1.0f, 0.0f);
		glVertex3f(-1.0f,  1.0f, -1.0f);	// Point 4 (Left)
		glEnd();
	}
}


/* Replaces gluPerspective. Sets the frustum to perspective mode.
 * fovY     - Field of vision in degrees in the y direction
 * aspect   - Aspect ratio of the viewport
 * zNear    - The near clipping distance
 * zFar     - The far clipping distance
*/
void perspectiveGL(GLdouble fovY, GLdouble aspect, GLdouble zNear, GLdouble zFar)
{
	GLdouble fW, fH;

	fH = tan( (fovY / 2) / 180 * M_PIl ) * zNear;
	fW = fH * aspect;

	glFrustum( -fW, fW, -fH, fH, zNear, zFar );
}


/* Calculate frame interval and print FPS each 5s */
int FrameTiming(void)
{
	Uint32 interval;
	static Uint32 current, last = 0, five = 0, nframes = 0;

	current = SDL_GetTicks();
	nframes++;

	if (current - five > 5*1000) {
		printf("%u frames in 5 seconds = %.1f FPS\n", nframes, (float)nframes/5.0f);
		nframes = 0;
		five = current;
	}
	
	interval = current - last;
	last = current;
	
	return interval;
}


/* Print errors from shaders and programs */
void printObjectInfoLog(GLuint obj)
{
	int infologLength = 0;
	int charsWritten  = 0;
	char *infoLog;

	if (glIsShader(obj) == GL_TRUE)
		glGetShaderiv(obj, GL_INFO_LOG_LENGTH, &infologLength);
	else
		glGetProgramiv(obj, GL_INFO_LOG_LENGTH, &infologLength);

	if (infologLength > 1) {
		infoLog = (char *)malloc(infologLength);
		if (glIsShader(obj) == GL_TRUE) {
			glGetShaderInfoLog(obj, infologLength, &charsWritten, infoLog);
			printf("Shader InfoLog: %s\n", infoLog);
		}
		else {
			glGetProgramInfoLog(obj, infologLength, &charsWritten, infoLog);
			printf("Program InfoLog: %s\n", infoLog);
		}
		free(infoLog);
	}
}
